package me.daryllkukas.apps.android.coolneonemote;

import java.util.ArrayList;

import android.app.Activity;
import android.bluetooth.BluetoothAdapter;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.util.Log;
import android.view.View;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.Button;
import android.widget.CompoundButton;
import android.widget.CompoundButton.OnCheckedChangeListener;
import android.widget.GridView;
import android.widget.TextView;
import android.widget.Toast;
import android.widget.ToggleButton;

public class MainActivity extends Activity implements OnCheckedChangeListener {

	GridView gridView;
	private ArrayList<ToggleButton> mToggleButtons = new ArrayList<ToggleButton>();
	private ToggleButton toggleButton;
	private TextView logview;
	private Button connect;
	private BluetoothAdapter mBluetoothAdapter = null;

	private String[] logArray = null;

	private BtInterface bt = null;

	static final String TAG = "Chihuahua";
	static final int REQUEST_ENABLE_BT = 3;

	// This handler listens to messages from the bluetooth interface and adds
	// them to the log
	final Handler handler = new Handler() {
		public void handleMessage(Message msg) {
			String data = msg.getData().getString("receivedData");
			addToLog(data);
		}
	};

	// this handler is dedicated to the status of the bluetooth connection
	final Handler handlerStatus = new Handler() {
		public void handleMessage(Message msg) {
			int status = msg.arg1;
			if (status == BtInterface.CONNECTED) {
				addToLog("Connected");
				connect.setText("Disconnect");
			} else if (status == BtInterface.DISCONNECTED) {
				addToLog("Disconnected");
				connect.setText("Connect");
			}
		}
	};

	// handles the log view modification
	// only the most recent messages are shown
	private void addToLog(String message) {
		for (int i = 1; i < logArray.length; i++) {
			logArray[i - 1] = logArray[i];
		}
		logArray[logArray.length - 1] = message;

		logview.setText("");
		for (int i = 0; i < logArray.length; i++) {
			if (logArray[i] != null) {
				logview.append(logArray[i] + "\n");
			}
		}
	}

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_main);
		for (int i = 0; i < 10; i++) {
			toggleButton = new ToggleButton(this);
			toggleButton.setLayoutParams(new GridView.LayoutParams(100, 100));
			String postionString = String.valueOf(i + 1);
			CharSequence textOff = "Neon " + postionString + " Off";
			CharSequence textOn = "Neon " + postionString + " On";
			toggleButton.setTextOff(textOff);
			toggleButton.setTextOn(textOn);
			toggleButton.setChecked(false);
			toggleButton.setText(textOff);
			toggleButton.setOnCheckedChangeListener(this);
			mToggleButtons.add(toggleButton);
		}

		logview = (TextView) findViewById(R.id.logview);
		// I chose to display only the last 3 messages
		logArray = new String[3];

		connect = (Button) findViewById(R.id.connect);
		connect.setOnClickListener(new View.OnClickListener() {
			
			@Override
			public void onClick(View v) {
				// TODO Auto-generated method stub
				if (BtInterface.isConnected) {
					bt.close();					
				} else {
					bt.connect();
				}
			}
		});

		gridView = (GridView) findViewById(R.id.switchGridView);

		gridView.setAdapter(new ButtonAdapter(mToggleButtons));

		gridView.setOnItemClickListener(new OnItemClickListener() {
			public void onItemClick(AdapterView<?> parent, View v,
					int position, long id) {
				// Send bt commands
				Toast.makeText(getApplicationContext(),
						((TextView) v).getText(), Toast.LENGTH_SHORT).show();
			}
		});
	}

	// it is better to handle bluetooth connection in onResume (ie able to reset
	// when changing screens)
	@Override
	public void onResume() {
		super.onResume();
		// first of all, we check if there is bluetooth on the phone
		mBluetoothAdapter = BluetoothAdapter.getDefaultAdapter();
		if (mBluetoothAdapter == null) {
			// Device does not support Bluetooth
			Log.v(TAG, "Device does not support Bluetooth");
		} else {
			// Device supports BT
			if (!mBluetoothAdapter.isEnabled()) {
				// if Bluetooth not activated, then request it
				Intent enableBtIntent = new Intent(
						BluetoothAdapter.ACTION_REQUEST_ENABLE);
				startActivityForResult(enableBtIntent, REQUEST_ENABLE_BT);
			} else {
				// BT activated, then initiate the BtInterface object to handle
				// all BT communication
				bt = new BtInterface(handlerStatus, handler);
			}
		}

		if (BtInterface.isConnected) {
			connect.setText("Disconnect");					
		} else {
			connect.setText("Connect");					
		}
	}

	// called only if the BT is not already activated, in order to activate it
	protected void onActivityResult(int requestCode, int resultCode,
			Intent moreData) {
		if (requestCode == REQUEST_ENABLE_BT) {
			if (resultCode == Activity.RESULT_OK) {
				// BT activated, then initiate the BtInterface object to handle
				// all BT communication
				bt = new BtInterface(handlerStatus, handler);
			} else if (resultCode == Activity.RESULT_CANCELED)
				Log.v(TAG, "BT not activated");
			else
				Log.v(TAG, "result code not known");
		} else {
			Log.v(TAG, "request code not known");
		}
	}

	@Override
	public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
		// TODO fix this logic
		int position = mToggleButtons.indexOf(buttonView);
		String[] letters = {"A","B","C","D","E","F","G","H","I","J","K","L","M","N","O","P","Q","R","S","T"};
		if (isChecked) {
			// Enable Lights
			bt.sendData(letters[position]);
			addToLog(String.valueOf(position + 1) + " is on");
		} else {
			// Disable Lights
			bt.sendData(letters[position + 10]);
			addToLog(String.valueOf(position + 1) + " is off");
		}
	}

}
